# coding: utf8
from __future__ import unicode_literals

import pytest
from ...tokens import Doc


@pytest.mark.parametrize('text,tag,lemma', [("inner", "JJ", "inner")])
def test_issue912(en_vocab, text, tag, lemma):
    '''Test base-forms of adjectives are preserved.'''
    doc = Doc(en_vocab, words=[text])
    doc[0].tag_ = tag
    assert doc[0].lemma_ == lemma




# coding: utf8
from __future__ import unicode_literals


# Stop words

STOP_WORDS = set("""
අතර
එච්චර
එපමණ
එලෙස
එවිට
ඒ
කට
කදී
කින්
ක්
ට
තුර
ත්
ද
නමුත්
නොහොත්
පමණ
පමණි
ම
මෙච්චර
මෙපමණ
මෙලෙස
මෙවිට
මේ
ය
යි
ලදී
ලෙස
වගේ
වන
විට
විටෙක
විතර
විය
වුව
වුවත්
වුවද
වූ
සමඟ
සහ
හා
හෙවත්
හෝ  
""".split())



# coding: utf8
from __future__ import unicode_literals

from ...symbols import NOUN, PROPN, PRON


def noun_chunks(obj):
    """
    Detect base noun phrases from a dependency parse. Works on both Doc and Span.
    """
    labels = ['nsubj', 'nsubj:pass', 'obj', 'iobj', 'ROOT', 'appos', 'nmod', 'nmod:poss']
    doc = obj.doc  # Ensure works on both Doc and Span.
    np_deps = [doc.vocab.strings[label] for label in labels]
    conj = doc.vocab.strings.add('conj')
    np_label = doc.vocab.strings.add('NP')
    seen = set()
    for i, word in enumerate(obj):
        if word.pos not in (NOUN, PROPN, PRON):
            continue
        # Prevent nested chunks from being produced
        if word.i in seen:
            continue
        if word.dep in np_deps:
            if any(w.i in seen for w in word.subtree):
                continue
            seen.update(j for j in range(word.left_edge.i, word.right_edge.i+1))
            yield word.left_edge.i, word.right_edge.i+1, np_label
        elif word.dep == conj:
            head = word.head
            while head.dep == conj and head.head.i < head.i:
                head = head.head
            # If the head is an NP, and we're coordinated to it, we're an NP
            if head.dep in np_deps:
                if any(w.i in seen for w in word.subtree):
                    continue
                seen.update(j for j in range(word.left_edge.i, word.right_edge.i+1))
                yield word.left_edge.i, word.right_edge.i+1, np_label


SYNTAX_ITERATORS = {
    'noun_chunks': noun_chunks
}



# coding: utf-8
from __future__ import unicode_literals

import pytest
from mock import Mock

from ..vocab import Vocab
from ..tokens import Doc, Span, Token
from ..tokens.underscore import Underscore


def test_create_doc_underscore():
    doc = Mock()
    doc.doc = doc
    uscore = Underscore(Underscore.doc_extensions, doc)
    assert uscore._doc is doc
    assert uscore._start is None
    assert uscore._end is None


def test_doc_underscore_getattr_setattr():
    doc = Mock()
    doc.doc = doc
    doc.user_data = {}
    Underscore.doc_extensions['hello'] = (False, None, None, None)
    doc._ = Underscore(Underscore.doc_extensions, doc)
    assert doc._.hello == False
    doc._.hello = True
    assert doc._.hello == True


def test_create_span_underscore():
    span = Mock(doc=Mock(), start=0, end=2)
    uscore = Underscore(Underscore.span_extensions, span,
                        start=span.start, end=span.end)
    assert uscore._doc is span.doc
    assert uscore._start is span.start
    assert uscore._end is span.end


def test_span_underscore_getter_setter():
    span = Mock(doc=Mock(), start=0, end=2)
    Underscore.span_extensions['hello'] = (None, None,
                                           lambda s: (s.start, 'hi'),
                                           lambda s, value: setattr(s, 'start',
                                                                    value))
    span._ = Underscore(Underscore.span_extensions, span,
                        start=span.start, end=span.end)

    assert span._.hello == (0, 'hi')
    span._.hello = 1
    assert span._.hello == (1, 'hi')


def test_token_underscore_method():
    token = Mock(doc=Mock(), idx=7, say_cheese=lambda token: 'cheese')
    Underscore.token_extensions['hello'] = (None, token.say_cheese,
                                            None, None)
    token._ = Underscore(Underscore.token_extensions, token, start=token.idx)
    assert token._.hello() == 'cheese'


@pytest.mark.parametrize('obj', [Doc, Span, Token])
def test_doc_underscore_remove_extension(obj):
    ext_name = 'to_be_removed'
    obj.set_extension(ext_name, default=False)
    assert obj.has_extension(ext_name)
    obj.remove_extension(ext_name)
    assert not obj.has_extension(ext_name)


@pytest.mark.parametrize('obj', [Doc, Span, Token])
def test_underscore_raises_for_dup(obj):
    obj.set_extension('test', default=None)
    with pytest.raises(ValueError):
        obj.set_extension('test', default=None)


@pytest.mark.parametrize('invalid_kwargs', [
    {'getter': None, 'setter': lambda: None},
    {'default': None, 'method': lambda: None, 'getter': lambda: None},
    {'setter': lambda: None},
    {'default': None, 'method': lambda: None},
    {'getter': True}])
def test_underscore_raises_for_invalid(invalid_kwargs):
    invalid_kwargs['force'] = True
    with pytest.raises(ValueError):
        Doc.set_extension('test', **invalid_kwargs)


@pytest.mark.parametrize('valid_kwargs', [
    {'getter': lambda: None},
    {'getter': lambda: None, 'setter': lambda: None},
    {'default': 'hello'},
    {'default': None},
    {'method': lambda: None}])
def test_underscore_accepts_valid(valid_kwargs):
    valid_kwargs['force'] = True
    Doc.set_extension('test', **valid_kwargs)



# coding: utf8
from __future__ import unicode_literals

from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS
from .stop_words import STOP_WORDS

from ..tokenizer_exceptions import BASE_EXCEPTIONS
from ...language import Language
from ...attrs import LANG
from ...util import update_exc


class IrishDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters[LANG] = lambda text: 'ga'

    tokenizer_exceptions = update_exc(BASE_EXCEPTIONS, TOKENIZER_EXCEPTIONS)
    stop_words = set(STOP_WORDS)

class Irish(Language):
    lang = 'ga'
    Defaults = IrishDefaults


__all__ = ['Irish']



# coding: utf8
from __future__ import unicode_literals
from ...attrs import LIKE_NUM

_num_words = set("""
صفر
واحد
إثنان
اثنان
ثلاثة
ثلاثه
أربعة
أربعه
خمسة
خمسه
ستة
سته
سبعة
سبعه
ثمانية
ثمانيه
تسعة
تسعه
ﻋﺸﺮﺓ
ﻋﺸﺮه
عشرون
عشرين
ثلاثون
ثلاثين
اربعون
اربعين
أربعون
أربعين
خمسون
خمسين
ستون
ستين
سبعون
سبعين
ثمانون
ثمانين
تسعون
تسعين
مائتين
مائتان
ثلاثمائة
خمسمائة
سبعمائة
الف
آلاف
ملايين
مليون
مليار
مليارات
""".split())

_ordinal_words = set("""
اول
أول
حاد
واحد
ثان
ثاني
ثالث
رابع
خامس
سادس
سابع
ثامن
تاسع
عاشر
""".split())


def like_num(text):
    """
    check if text resembles a number
    """
    text = text.replace(',', '').replace('.', '')
    if text.isdigit():
        return True
    if text.count('/') == 1:
        num, denom = text.split('/')
        if num.isdigit() and denom.isdigit():
            return True
    if text in _num_words:
        return True
    if text in _ordinal_words:
        return True
    return False


LEX_ATTRS = {
    LIKE_NUM: like_num
}



# coding: utf8
from __future__ import unicode_literals

from .._messages import Messages
from ...compat import json_dumps, path2str
from ...util import prints
from ...gold import iob_to_biluo
import re


def conllu2json(input_path, output_path, n_sents=10, use_morphology=False):

    """
    Convert conllu files into JSON format for use with train cli.
    use_morphology parameter enables appending morphology to tags, which is
    useful for languages such as Spanish, where UD tags are not so rich.
    """
    # by @dvsrepo, via #11 explosion/spacy-dev-resources

    """     
    Extract NER tags if available and convert them so that they follow
    BILUO and the Wikipedia scheme
    """
    # by @katarkor

    docs = []
    sentences = []
    conll_tuples = read_conllx(input_path, use_morphology=use_morphology)
    checked_for_ner = False
    has_ner_tags = False

    for i, (raw_text, tokens) in enumerate(conll_tuples):
        sentence, brackets = tokens[0]
        if not checked_for_ner:
            has_ner_tags = is_ner(sentence[5][0])
            checked_for_ner = True
        sentences.append(generate_sentence(sentence, has_ner_tags))
        # Real-sized documents could be extracted using the comments on the
        # conluu document

        if(len(sentences) % n_sents == 0):
            doc = create_doc(sentences, i)
            docs.append(doc)
            sentences = []

    output_filename = input_path.parts[-1].replace(".conll", ".json")
    output_filename = input_path.parts[-1].replace(".conllu", ".json")
    output_file = output_path / output_filename
    with output_file.open('w', encoding='utf-8') as f:
        f.write(json_dumps(docs))
    prints(Messages.M033.format(n_docs=len(docs)),
           title=Messages.M032.format(name=path2str(output_file)))


def is_ner(tag):

    """ 
    Check the 10th column of the first token to determine if the file contains
    NER tags 
    """

    tag_match = re.match('([A-Z_]+)-([A-Z_]+)', tag)
    if tag_match:
        return True
    elif tag == "O":
        return True
    else:
        return False

def read_conllx(input_path, use_morphology=False, n=0):
    text = input_path.open('r', encoding='utf-8').read()
    i = 0
    for sent in text.strip().split('\n\n'):
        lines = sent.strip().split('\n')
        if lines:
            while lines[0].startswith('#'):
                lines.pop(0)
            tokens = []
            for line in lines:

                parts = line.split('\t')
                id_, word, lemma, pos, tag, morph, head, dep, _1, iob = parts
                if '-' in id_ or '.' in id_:
                    continue
                try:
                    id_ = int(id_) - 1
                    head = (int(head) - 1) if head != '0' else id_
                    dep = 'ROOT' if dep == 'root' else dep
                    tag = pos if tag == '_' else tag
                    tag = tag+'__'+morph  if use_morphology else tag
                    tokens.append((id_, word, tag, head, dep, iob))
                except:
                    print(line)
                    raise
            tuples = [list(t) for t in zip(*tokens)]
            yield (None, [[tuples, []]])
            i += 1
            if n >= 1 and i >= n:
                break

def simplify_tags(iob):
   
    """
    Simplify tags obtained from the dataset in order to follow Wikipedia
    scheme (PER, LOC, ORG, MISC). 'PER', 'LOC' and 'ORG' keep their tags, while
    'GPE_LOC' is simplified to 'LOC', 'GPE_ORG' to 'ORG' and all remaining tags to
    'MISC'.     
    """

    new_iob = []
    for tag in iob:
        tag_match = re.match('([A-Z_]+)-([A-Z_]+)', tag)
        if tag_match:
            prefix = tag_match.group(1)
            suffix = tag_match.group(2)
            if suffix == 'GPE_LOC':
                suffix = 'LOC'
            elif suffix == 'GPE_ORG':
                suffix = 'ORG'
            elif suffix != 'PER' and suffix != 'LOC' and suffix != 'ORG':
                suffix = 'MISC'
            tag = prefix + '-' + suffix
        new_iob.append(tag)
    return new_iob

def generate_sentence(sent, has_ner_tags):
    (id_, word, tag, head, dep, iob) = sent
    sentence = {}
    tokens = []
    if has_ner_tags:
        iob = simplify_tags(iob)
        biluo = iob_to_biluo(iob)
    for i, id in enumerate(id_):
        token = {}
        token["id"] = id
        token["orth"] = word[i]
        token["tag"] = tag[i]
        token["head"] = head[i] - id
        token["dep"] = dep[i]
        if has_ner_tags:
            token["ner"] = biluo[i]
        tokens.append(token)
    sentence["tokens"] = tokens
    return sentence


def create_doc(sentences,id):
    doc = {}
    paragraph = {}
    doc["id"] = id
    doc["paragraphs"] = []
    paragraph["sentences"] = sentences
    doc["paragraphs"].append(paragraph)
    return doc



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.ro import Romanian
>>> from spacy.lang.ro.examples import sentences
>>> nlp = Romanian()
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "Apple plănuiește să cumpere o companie britanică pentru un miliard de dolari",
    "Municipalitatea din San Francisco ia în calcul interzicerea roboților curieri pe trotuar",
    "Londra este un oraș mare în Regatul Unit",
    "Unde ești?",
    "Cine este președintele Franței?",
    "Care este capitala Statelor Unite?",
    "Când s-a născut Barack Obama?"
]



# encoding: utf8
# data from Korakot Chaovavanich (https://www.facebook.com/photo.php?fbid=390564854695031&set=p.390564854695031&type=3&permPage=1&ifg=1)
from __future__ import unicode_literals

from ...symbols import POS, NOUN, PRON, ADJ, ADV, INTJ, PROPN, DET, NUM, AUX
from ...symbols import ADP, CCONJ, PART, PUNCT, SPACE, SCONJ
TAG_MAP = {
    # NOUN
    "NOUN":     {POS: NOUN},
    "NCMN":     {POS: NOUN},
    "NTTL":     {POS: NOUN},
    "CNIT":     {POS: NOUN},
    "CLTV":     {POS: NOUN},
    "CMTR":     {POS: NOUN},
    "CFQC":     {POS: NOUN},
    "CVBL":     {POS: NOUN},
    # PRON
    "PRON":     {POS: PRON},
    "NPRP":     {POS: PRON},
    # ADJ
    "ADJ":      {POS: ADJ},
    "NONM":     {POS: ADJ},
    "VATT":     {POS: ADJ},
    "DONM":     {POS: ADJ},
    # ADV
    "ADV":      {POS: ADV},
    "ADVN":     {POS: ADV},
    "ADVI":     {POS: ADV},
    "ADVP":     {POS: ADV},
    "ADVS":     {POS: ADV},
    # INT
    "INT":      {POS: INTJ},
    # PRON
    "PROPN":    {POS: PROPN},
    "PPRS":     {POS: PROPN},
    "PDMN":     {POS: PROPN},
    "PNTR":     {POS: PROPN},
    # DET
    "DET":      {POS: DET},
    "DDAN":     {POS: DET},
    "DDAC":     {POS: DET},
    "DDBQ":     {POS: DET},
    "DDAQ":     {POS: DET},
    "DIAC":     {POS: DET},
    "DIBQ":     {POS: DET},
    "DIAQ":     {POS: DET},
    "DCNM":     {POS: DET},
    # NUM
    "NUM":      {POS: NUM},
    "NCNM":     {POS: NUM},
    "NLBL":     {POS: NUM},
    "DCNM":     {POS: NUM},
    # AUX
    "AUX":      {POS: AUX},
    "XVBM":     {POS: AUX},
    "XVAM":     {POS: AUX},
    "XVMM":     {POS: AUX},
    "XVBB":     {POS: AUX},
    "XVAE":     {POS: AUX},
    # ADP
    "ADP":      {POS: ADP},
    "RPRE":     {POS: ADP},
    # CCONJ
    "CCONJ":    {POS: CCONJ},
    "JCRG":     {POS: CCONJ},
    # SCONJ
    "SCONJ":    {POS: SCONJ},
    "PREL":     {POS: SCONJ},
    "JSBR":     {POS: SCONJ},
    "JCMP":     {POS: SCONJ},
    # PART
    "PART":     {POS: PART},
    "FIXN":     {POS: PART},
    "FIXV":     {POS: PART},
    "EAFF":     {POS: PART},
    "AITT":     {POS: PART},
    "NEG":      {POS: PART},
    # PUNCT
    "PUNCT":    {POS: PUNCT},
    "PUNC":     {POS: PUNCT},
    "_SP":      {POS: SPACE}
}



import spacy
import spacy.lang.en
from spacy.pipeline import TextCategorizer

def test_bytes_serialize_issue_1105():
    nlp = spacy.lang.en.English()
    tokenizer = nlp.tokenizer
    textcat = TextCategorizer(tokenizer.vocab, labels=['ENTITY', 'ACTION', 'MODIFIER'])
    textcat_bytes = textcat.to_bytes()
